@@ -121,6 +121,9 @@ UnityScreen::OnLauncherHiddenChanged ()
 void
 UnityScreen::paintPanelShadow (const GLMatrix &matrix)
 {
+  if (relayoutSourceId > 0)
+    return;
+    
   if (PluginAdapter::Default ()->IsExpoActive ())
     return;
   
@@ -183,7 +186,7 @@ UnityScreen::paintDisplay (const CompRegion &region)
   
   wt->RenderInterfaceFromForeignCmd (&geo);
   nuxEpilogue ();
-  
+
   doShellRepaint = false;
 }
 
@@ -673,11 +676,26 @@ UnityWindow::resizeNotify (int x, int y, int w, int h)
   window->resizeNotify (x, y, w, h);
 }
 
+CompPoint
+UnityWindow::tryNotIntersectLauncher (CompPoint &pos)
+{
+  UnityScreen *us = UnityScreen::get (screen);
+  nux::Geometry geo = us->launcher->GetAbsoluteGeometry ();
+  CompRect launcherGeo (geo.x, geo.y, geo.width, geo.height);
+
+  if (launcherGeo.contains (pos))
+  {
+    if (screen->workArea ().contains (CompRect (launcherGeo.right () + 1, pos.y (), window->width (), window->height ())))
+      pos.setX (launcherGeo.right () + 1);
+  }
+  
+  return pos;
+}
+
 bool
 UnityWindow::place (CompPoint &pos)
 {
   UnityScreen *us = UnityScreen::get (screen);
-  nux::Geometry geo = us->launcher->GetAbsoluteGeometry ();
   Launcher::LauncherHideMode hideMode = us->launcher->GetHideMode ();
   
   bool result = window->place (pos);
@@ -686,11 +704,9 @@ UnityWindow::place (CompPoint &pos)
   {
     case Launcher::LAUNCHER_HIDE_DODGE_WINDOWS:
     case Launcher::LAUNCHER_HIDE_DODGE_ACTIVE_WINDOW:
-      if (pos.x () <= geo.width && window->width () + geo.width + 1< screen->workArea ().width ())
-        {
-          pos.setX (geo.width + 1);
-        }
+      pos = tryNotIntersectLauncher (pos);
       break;
+    
     default:
       break;
   }
@@ -795,7 +811,27 @@ UnityScreen::Relayout ()
   gdk_screen_get_monitor_geometry (scr, primary_monitor, &rect);
   _primary_monitor = rect;
 
-  wt->SetWindowSize (rect.width, rect.height);
+
+  int display_x = 0;
+  int display_y = 0;
+  int display_w = 0;
+  int display_h = 0;
+
+  UScreen* uscreen = UScreen::GetDefault();
+  std::vector<nux::Geometry>& monitors = uscreen->GetMonitors ();
+  for (unsigned int i= 0; i < monitors.size(); ++i)
+  {
+      display_x = nux::Min<int>(display_x, monitors[i].x);
+      display_y = nux::Min<int>(display_y, monitors[i].y);
+      display_w = nux::Max<int>(display_w, monitors[i].x + monitors[i].width);
+      display_h = nux::Max<int>(display_h, monitors[i].y + monitors[i].height);
+  }
+
+  display_w -= display_x;
+  display_h -= display_y;
+  
+
+  nux::GetWindowThread()->SetWindowSize(display_w, display_h);
 
   lCurGeom = launcherWindow->GetGeometry(); 
   launcher->SetMaximumHeight(rect.height - panel_height);
@@ -825,6 +861,8 @@ UnityScreen::RelayoutTimeout (gpointer data)
   uScr->NeedsRelayout ();
   uScr->Relayout();
   uScr->relayoutSourceId = 0;
+  
+  uScr->cScreen->damageScreen ();
 
   return FALSE;
 }
@@ -854,20 +892,10 @@ write_logger_data_to_disk (gpointer data)
   return FALSE;
 }
 
-void
-OnMonitorChanged (GdkScreen* screen,
-                  gpointer   data)
+void 
+UnityScreen::outputChangeNotify ()
 {
-  UnityScreen* uscreen = (UnityScreen*) data;
-  uscreen->ScheduleRelayout (500);
-}
-
-void
-OnSizeChanged (GdkScreen* screen,
-               gpointer   data)
-{
-  UnityScreen* uscreen = (UnityScreen*) data;
-  uscreen->ScheduleRelayout (500);
+  ScheduleRelayout (500);
 }
 
 UnityScreen::UnityScreen (CompScreen *screen) :
@@ -964,15 +992,6 @@ UnityScreen::UnityScreen (CompScreen *screen) :
   g_timeout_add (0, &UnityScreen::initPluginActions, this);
   g_timeout_add (5000, (GSourceFunc) write_logger_data_to_disk, NULL);
 
-  g_signal_connect (gdk_screen_get_default (),
-                    "monitors-changed",
-                     G_CALLBACK (OnMonitorChanged),
-                     this);
-  g_signal_connect (gdk_screen_get_default (),
-                    "size-changed",
-                    G_CALLBACK (OnSizeChanged),
-                    this);
-
   GeisAdapter::Default (screen)->Run ();
   gestureEngine = new GestureEngine (screen);
   
@@ -989,7 +1008,6 @@ UnityScreen::~UnityScreen ()
   delete placesController;
   panelController->UnReference ();
   delete controller;
-  layout->UnReference ();
   launcher->UnReference ();
   launcherWindow->UnReference ();
 
@@ -1017,25 +1035,31 @@ void UnityScreen::initLauncher (nux::NThread* thread, void* InitData)
 
   LOGGER_START_PROCESS ("initLauncher-Launcher");
   self->launcherWindow = new nux::BaseWindow(TEXT("LauncherWindow"));
+  self->launcherWindow->SinkReference ();
+
   self->launcher = new Launcher(self->launcherWindow, self->screen);
+  self->launcher->SinkReference ();
+
   self->launcher->hidden_changed.connect (sigc::mem_fun (self, &UnityScreen::OnLauncherHiddenChanged));
   
+
   self->AddChild (self->launcher);
 
-  self->layout = new nux::HLayout();
-  self->layout->AddView(self->launcher, 1);
-  self->layout->SetContentDistribution(nux::eStackLeft);
-  self->layout->SetVerticalExternalMargin(0);
-  self->layout->SetHorizontalExternalMargin(0);
+  nux::HLayout* layout = new nux::HLayout();
+  layout->AddView(self->launcher, 1);
+  layout->SetContentDistribution(nux::eStackLeft);
+  layout->SetVerticalExternalMargin(0);
+  layout->SetHorizontalExternalMargin(0);
 
   self->controller = new LauncherController (self->launcher, self->screen, self->launcherWindow);
 
   self->launcherWindow->SetConfigureNotifyCallback(&UnityScreen::launcherWindowConfigureCallback, self);
-  self->launcherWindow->SetLayout(self->layout);
+  self->launcherWindow->SetLayout(layout);
   self->launcherWindow->SetBackgroundColor(nux::Color(0x00000000));
   self->launcherWindow->ShowWindow(true);
   self->launcherWindow->EnableInputWindow(true, "launcher", false, false);
   self->launcherWindow->InputWindowEnableStruts(true);
+  
   self->launcherWindow->SetEnterFocusInputArea (self->launcher);
 
   /* FIXME: this should not be manual, should be managed with a
@@ -1071,6 +1095,8 @@ void UnityScreen::initLauncher (nux::NThread* thread, void* InitData)
   self->launcher->SetLaunchAnimation (Launcher::LAUNCH_ANIMATION_PULSE);
   self->launcher->SetUrgentAnimation (Launcher::URGENT_ANIMATION_WIGGLE);
   self->ScheduleRelayout (0);
+  
+  self->OnLauncherHiddenChanged ();
 
   END_FUNCTION ();
 }
